using System;
using System.Collections.Generic;
using System.Configuration;
using SimpleJSON;
using MacGruber;
using UnityEngine;

namespace CheesyFX
{
    public class FlexIt : MVRScript
    {
        private static string morphPath = "Custom/Atom/Person/Morphs/female/CheesyFX/FlexIt/";
        private List<FlexRegion> flexRegions = new List<FlexRegion>();
        private static float _speed = 10f;
        public static float speed  = _speed * Time.fixedDeltaTime;
        
        private static GenerateDAZMorphsControlUI morphUI;
        private UIDynamicTabBar regionTabbar;
        private JSONStorableString legend = new JSONStorableString("legend", "");

        private string[] regionNames = { "Knee", "Thigh", "Abdomen", "Shoulder"};
        private string[][] rbNames = { new[]{"Shin"}, new[]{ "Thigh" }, new[]{ "pelvis", "chest" }, new[]{ "Shldr" }};

        public void OnEnable()
        {
            for (int i = 0; i < flexRegions.Count; i++)
            {
                flexRegions[i].enabled = true;
                // flexRegions[i].OnDisable();
            }
        }

        public override void Init()
        {
            DAZCharacterSelector dcs = containingAtom.GetComponentInChildren<DAZCharacterSelector>();
            if (dcs.selectedCharacter.isMale)
            {
                morphPath = morphPath.Replace("female", "male");
            }
            if (Utils.IsInPackage(this))
            {
                var packageUid = Utils.GetPluginPath(this).Split(':')[0] + ":/";
                // factoryDefaults = packageUid + factoryDefaults;
                morphPath = packageUid + morphPath;
            }
            morphUI = dcs.morphsControlUI;
            
            for(int i=0; i<regionNames.Length; i++)
            {
                var flexRegion = gameObject.AddComponent<FlexRegion>();
                flexRegion.Init(i, regionNames[i], rbNames[i], i!=2);
                flexRegions.Add(flexRegion);
            }
            
            UIManager.OnInitUI(CreateUIElement, leftUIElements, rightUIElements);
            CreateUI();
        }
        
        private UIDynamicV4Slider sRatioUid;
        private UIDynamicV4Slider aStrengthUid;
        private UIDynamicV4Slider gStrengthUid;
        private UIDynamicV4Slider minAccelUid;
        private UIDynamicV4Slider maxMorphUid;
        private UIDynamicTVector4Legend v4legend;
        private void CreateUI()
        {
            regionTabbar = UIManager.CreateTabBar(regionNames, val => RegisterVectors(), script:this, invokeDefault:false);

            v4legend = UIManager.createVector4Legend(this);
            maxMorphUid = UIManager.CreateV4Slider(script: this, fontSize:25);
            sRatioUid = UIManager.CreateV4Slider(script: this, constrained:true, fontSize:25);
            aStrengthUid = UIManager.CreateV4Slider(script: this, fontSize:25);

            System.Text.StringBuilder sb = new System.Text.StringBuilder();
            sb.AppendLine("<b>Amount:</b>");
            sb.AppendLine("Sets the maximum morph value achievable and scales the dynamic effects.");
            sb.AppendLine("");
            sb.AppendLine("<b>S-Ratio:</b>");
            sb.AppendLine("Sets which portion of the max morph value can be achieved in static poses. If <1, full morphing is only achievable through dynamic effects (angular acceleration).");
            sb.AppendLine("0 means only dynamic effects and 1 leads to pose based morphing only.");
            sb.AppendLine("");
            sb.AppendLine("<b>Dynamic</b>");
            sb.AppendLine("The strength of the dynamic effects. Gets multiplied with 'Amount'.");
            Utils.SetupInfoText(this, sb.ToString(), 580f, false);
            // gStrengthUid = UIManager.CreateV4Slider(script: this);
            // minAccelUid = UIManager.CreateV4Slider(script: this);
            
            
            maxMorphUid.setToggleCallbackFunction += maxMorphUid.ToggleSync;
            maxMorphUid.toggle.label = "Sync";
            sRatioUid.setToggleCallbackFunction += sRatioUid.ToggleSync;
            sRatioUid.toggle.label = "Sync";
            aStrengthUid.setToggleCallbackFunction += aStrengthUid.ToggleSync;
            aStrengthUid.toggle.label = "Sync";
            
            regionTabbar.SelectTab(0);
        }

        private void RegisterVectors()
        {
            var flexRegion = flexRegions[regionTabbar.id];
            string[] suffixes = new string[4];
            System.Text.StringBuilder sb = new System.Text.StringBuilder();
            sb.AppendLine($"<b>{flexRegion.name}:</b>");
            switch (regionTabbar.id)
            {
                case 0:
                    v4legend.SetLabels(new []{"Straighten", "Bend", "", ""});
                    break;
                case 1:
                    v4legend.SetLabels(new []{"Forward", "Backwards", "Inwards", "Outwards"});
                    break;
                case 2:
                    v4legend.SetLabels(new []{"Crunch", "Straighten", "Crunch right", "Crunch left"});
                    break;
                case 3:
                    v4legend.SetLabels(new []{"Forward", "Backwards", "Down", "Up"});
                    break;
            }
            legend.val = sb.ToString();
            
            sRatioUid.RegisterVector(flexRegion.sRatioJ);
            aStrengthUid.RegisterVector(flexRegion.aStrengthJ);
            maxMorphUid.RegisterVector(flexRegion.amountJ);
            

            for (int i = 0; i < 4; i++)
            {
                bool active = i < flexRegion.morphsCount;
                sRatioUid.SliderSetActive(i, active);
                aStrengthUid.SliderSetActive(i, active);
                maxMorphUid.SliderSetActive(i, active);
            }
            // flexRegion.sRatioJ.constrained = true;
            // sRatioUid.SetConstrained(true);
            

        }

        public void FixedUpdate()
        {
            speed = _speed * Time.fixedDeltaTime;
        }

        public void OnDisable()
        {
            for (int i = 0; i < flexRegions.Count; i++)
            {
                flexRegions[i].enabled = false;
            }
        }
        
        public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
        {
            for (int i = 0; i < flexRegions.Count; i++)
            {
                flexRegions[i].Reset();
            }
            JSONClass jc = base.GetJSON(includePhysical, includeAppearance, needsStore);
            return jc;
        }

        public static DAZMorph GetMorphByUid(string name)
        {
            DAZMorph morph;
            string uid = morphPath + name +".vmi";
            morph = morphUI.GetMorphByUid(uid);
            // if(morph == null) $"Morph '{uid}' not found!".Print();
            return morph;
        }
    }
}