using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Policy;
using UnityEngine;

namespace CheesyFX
{
    public class FlexRegion : MonoBehaviour
    {
        private RBDynamics[] rbDynamics;
        private RBDynamics rbDynamicAlt;
        private int id;
        public string name;
        private List<List<DAZMorph>> morphs = new List<List<DAZMorph>>();
        private bool[] doDyn = {true,true,true,true};
        private bool[] doStatic = {true,true,true,true};
        public int morphsCount => morphs.Count;

        public JSONStorableVector4 aStrengthJ = new JSONStorableVector4(
            "Dynamic",
            new Vector4(2f, 2f, 2f, 2f),
            new Vector4(),
            new Vector4(10f,10f,10f, 10f));
        public JSONStorableVector4 amountJ = new JSONStorableVector4(
            "Amount",
            new Vector4(2f, 2f, 2f, 2f),
            new Vector4(),
            new Vector4(5f,5f,5f, 5f));
        public JSONStorableVector4 minAccelJ = new JSONStorableVector4(
            "minAcc",
            new Vector4(1f, 1f, 1f, 1f),
            new Vector4(),
            new Vector4(20f,20f,20f, 20f));
        
        public JSONStorableVector4 sRatioJ = new JSONStorableVector4(
            "S-Ratio",
            new Vector4(.75f,.75f,.75f, .75f),
            new Vector4(0f, 0f, 0f, 0f),
            new Vector4(1f,1f,1f, 1f));
        
        Transform world = SuperController.singleton.transform;
        private Dictionary<int, string> sides = new Dictionary<int, string>
        {
            { 0, "L" }, { 1, "R" }
        };
        private Dictionary<int, string> directions = new Dictionary<int, string>
        {
            { 0, "Fwd" }, { 1, "Bwd" }, { 2, "In" }, { 3, "Out" }
        };

        private Vector4 sRatio { get; set; }
        private Vector4 aStrength { get; set; }
        private Vector4 gStrength { get; set; }
        private Vector4 minAccel { get; set; }
        // private Vector4 maxAccel { get; set; }
        private Vector4 amount { get; set; }

        private Func<int, int, float> GetAcceleration;
        // private Func<int, float> GetFlex;


        public void Init(int id, string region, string[] rbNames, bool sym = true)
        {
            this.id = id;
            name = region;
            var atom = gameObject.GetAtom();
            Rigidbody rb;
            if (sym)
            {
                rbDynamics = new RBDynamics[2];
                for (int i = 0; i < 2; i++)
                {
                    string side = sides[i];
                    rb = atom.forceReceivers.FirstOrDefault(x => x.name == side.ToLower() + rbNames[0])
                        ?.GetComponent<Rigidbody>();
                    rbDynamics[i] = rb.AddRbDynamic(i);
                }
            }
            else
            {
                rbDynamics = new RBDynamics[1];
                rb = atom.forceReceivers.FirstOrDefault(x => x.name == rbNames[0])
                    ?.GetComponent<Rigidbody>();
                rbDynamics[0] = rb.AddRbDynamic();
            }


            if (rbNames.Length == 2)
            {
                rb = atom.forceReceivers.FirstOrDefault(x => x.name == rbNames[1])
                    ?.GetComponent<Rigidbody>();
                rbDynamicAlt = rb.AddRbDynamic();
                GetAcceleration = (directionId, sideId) =>
                {
                    float val;
                    if (directionId < 2) val = rbDynamics[sideId].angularAcceleration.x - .75f*rbDynamicAlt.angularAcceleration.x;
                    else val = rbDynamics[sideId].angularAcceleration.z + .75f*rbDynamicAlt.angularAcceleration.z;
                    val *= amount[directionId];
                    return val;
                };
            }
            else
            {
                if(id == 3)
                    GetAcceleration = (directionId, sideId) =>
                    {
                        float val;
                        if (directionId < 2) val = -rbDynamics[sideId].angularAcceleration.y;
                        else val = -rbDynamics[sideId].angularAcceleration.z;
                        val *= amount[directionId];
                        return val;
                    };
                else
                    GetAcceleration = (directionId, sideId) =>
                    {
                        float val;
                        if (directionId < 2) val = rbDynamics[sideId].angularAcceleration.x;
                        else val = rbDynamics[sideId].angularAcceleration.z;
                        val *= amount[directionId];
                        return val;
                    };
            }

            for (int i = 0; i < 4; i++)
            {
                bool morphsFound = false;
                List<DAZMorph> sideMorphs = new List<DAZMorph>();
                if(sym)
                {
                    for (int j = 0; j < 2; j++)
                    {
                        string side = sides[j];
                        var morph = FlexIt.GetMorphByUid($"FI_{region}{directions[i]}{side}");
                        if (morph == null) continue;
                        sideMorphs.Add(morph);
                        morph.jsonFloat.constrained = true;
                        morphsFound = true;
                    }
                }
                else
                {
                    var morph = FlexIt.GetMorphByUid($"FI_{region}{directions[i]}");
                    if (morph == null) continue;
                    sideMorphs.Add(morph);
                    morph.jsonFloat.constrained = true;
                    morphsFound = true;
                }
                if(morphsFound) morphs.Add(sideMorphs);
            }

            aStrengthJ.setCallbackFunction += val => aStrength = val;
            minAccelJ.setCallbackFunction += val => minAccel = val;
            sRatioJ.setCallbackFunction += val =>
            {
                doDyn = new[] {val[0] < 1f, val[1] < 1f, val[2] < 1f, val[3] < 1f};
                doStatic = new[] {val[0] > 0f, val[1] > 0f, val[2] > 0f, val[3] > 0f};
                gStrength = FlexIt.speed * MultiplyV4(val, amount);
            };
            amountJ.setCallbackFunction += val =>
            {
                amount = val;
                gStrength = FlexIt.speed * MultiplyV4(sRatio, val);
                for (int i = 0; i < morphs.Count; i++)
                {
                    for(int j=0; j<morphs[i].Count; j++) morphs[i][j].jsonFloat.max = val[i];
                }
            };
            SetupDefaults(id);
            sRatioJ.constrained = false;
        }

        private void SetupDefaults(int id)
        {
            Vector4 aStrengthD = aStrengthJ.defaultVal;
            Vector4 minAccelD = minAccelJ.defaultVal;
            Vector4 sdRatioD = sRatioJ.defaultVal;
            Vector4 maxMorphD = amountJ.defaultVal;
            switch (id)
            {
                case 0:
                    maxMorphD.x = 3f;
                    break;
                case 1:
                    aStrengthD.x = 2f;
                    aStrengthD.y = .5f;
                    break;
                case 2:
                    maxMorphD = new Vector4(2f, 3f, 2.5f, 2.5f);
                    aStrengthD = new Vector4(.15f, .25f, 1.5f, 1.5f);
                    sdRatioD.x = .5f;
                    break;
                case 3:
                    aStrengthD = new Vector4(1f, 1f, .5f, 1f);
                    break;
            }

            aStrengthJ.val = aStrengthJ.defaultVal = aStrengthD;
            minAccel = minAccelJ.val = minAccelJ.defaultVal = minAccelD;
            sRatio = sRatioJ.val = sRatioJ.defaultVal = sdRatioD;
            amount = amountJ.val = amountJ.defaultVal = maxMorphD;
            aStrength = aStrengthJ.defaultVal;
            
            var val = amountJ.defaultVal;
            amount = val;
            gStrength = FlexIt.speed * MultiplyV4(sRatio, amount);
            for (int i = 0; i < morphs.Count; i++)
            {
                for(int j=0; j<morphs[i].Count; j++) morphs[i][j].jsonFloat.max = val[i];
            }
        }

        public void FixedUpdate()
        {
            float speed = FlexIt.speed;
            for (int i = 0; i < morphs.Count; i++)
            {
                if(amount[i] == 0f) continue;
                bool dyn = doDyn[i];
                bool stat = doStatic[i];
                for (int j = 0; j < morphs[i].Count; j++)
                {
                    float m = morphs[i][j].morphValue;
                    if (m > .001f) morphs[i][j].morphValue = Mathf.Lerp(m, 0f, speed);
                    if(dyn)
                    {
                        float f = GetAcceleration(i, j);
                        bool inRange = (i == 1 || i == 3) ? f > minAccel[i] : f < -minAccel[i];
                        if (inRange)
                        {
                            f = Mathf.Abs(f) * aStrength[i] * Time.fixedDeltaTime;
                            f += morphs[i][j].morphValue;
                            morphs[i][j].morphValue = f;
                        }
                    }
                    if(stat)
                    {
                        if (i < 2)
                        {
                            float flex = rbDynamics[j].rigidbody.transform.forward.y;
                            bool flexed = i == 0 ? flex > 0f : flex < 0f;
                            if (flexed) morphs[i][j].morphValue += Mathf.Abs(flex) * gStrength[i];
                        }
                        else
                        {
                            float flex = rbDynamics[j].rigidbody.transform.right.y;
                            bool flexed = i == 2 ? flex > 0f : flex < 0f;
                            if (j == 1) flexed = !flexed;
                            if (flexed) morphs[i][j].morphValue += Mathf.Abs(flex) * gStrength[i];
                        }
                    }
                }
            }
        }

        public void Reset()
        {
            
            for (int i = 0; i < morphs.Count; i++)
            {
                for(int j=0; j<morphs[i].Count; j++) morphs[i][j].morphValue = 0f;
            }
        }

        public void OnDisable()
        {
            for (int i = 0; i < rbDynamics.Length; i++) rbDynamics[i].uses--;
            Reset();
        }

        private Vector4 MultiplyV4(Vector4 a, Vector4 b)
        {
            for (int i = 0; i < 4; i++) a[i] *= b[i];
            return a;
        }
    }
}